//Software base para envio de dados para Broker via MQTT - OPS150
//Leituras das portas digitais e analógicas
//Desenvolvido por Isaias Marcondes 
//Opller

#include <PubSubClient.h>
#include <WiFi.h>

//Dados do Broker - MQTT
const char* broker = " "; //insira entre " " o endereço do broker 
uint16_t port = 1883; //insira a porta do Broker 
const char* username = " "; //insira entre " " o username caso possua
const char* password = " "; //insira entre " " a senha caso possua
const char* clientid = " "; //insira entre " " o client

//Definição das labels
const char * VARIABLE_LABEL_1  = "digital1"; 
const char * VARIABLE_LABEL_2  = "digital2"; 
const char * VARIABLE_LABEL_3  = "digital3"; 
const char * VARIABLE_LABEL_4  = "digital4";  
const char * VARIABLE_LABEL_5  = "digital5"; 
const char * VARIABLE_LABEL_6  = "digital6"; 
const char * VARIABLE_LABEL_7  = "digital7";  
const char * VARIABLE_LABEL_8  = "digital8";  
const char * VARIABLE_LABEL_9  = "digital9"; 
const char * VARIABLE_LABEL_10 = "digital10";  
const char * VARIABLE_LABEL_11 = "analogic1"; 
const char * VARIABLE_LABEL_12 = "analogic2";
const char * VARIABLE_LABEL_13 = "analogic3"; 

//Pinos das entradas digitais
#define pin_Latch165 15
#define pin_Clock165 4
#define pin_Data165 5
#define pin_X9 23
#define pin_X10 35

//Pinos das entradas analógicas
#define pin_AI1 34
#define pin_AI2 39
#define pin_AI3 36

const char *SSID = " "; //Insira entre " " a rede wifi
const char *PASSWORD = " "; //Insira entre " " a senha wifi

unsigned long TMR_POST;

/*Variaveis para envio dos dados*/
char payload[300];
char topic[150];

int Digital_in [10];
int Analog_in [3];

WiFiClient OPS150;
PubSubClient mqtt(OPS150);

long lastReconnectAttempt = 0;

void setup() {
  
  Serial.begin(115200);

  pinMode(pin_Latch165, OUTPUT);
  pinMode(pin_Clock165, OUTPUT);
  pinMode(pin_Data165, INPUT);
  pinMode(pin_X9, INPUT);
  pinMode(pin_X10, INPUT);

 WiFi.begin(SSID, PASSWORD);
 while (WiFi.status() != WL_CONNECTED) {
 delay(500);
 Serial.println("Connecting to WiFi..");
 }
 Serial.print("Conectado em ");
 Serial.println(SSID);

  // MQTT Broker setup
  mqtt.setServer(broker, port);
  mqtt.setCallback(mqttCallback);
}

boolean mqttConnect() {
  Serial.print("Connecting to ");
  Serial.print(broker);

  //Autenticação
  boolean status = mqtt.connect(clientid, username, password);

  if (status == false) {
    Serial.println(" fail");
    return false;
  }
  Serial.println(" OK");
  return mqtt.connected();
}

void loop() {

  //COnectando com o Broker
  if (!mqtt.connected()) {
    Serial.println("=== MQTT NOT CONNECTED ===");
    // Reconnect every 10 seconds
    unsigned long t = millis();
    if (t - lastReconnectAttempt > 10000L) {
      lastReconnectAttempt = t;
      if (mqttConnect()) {
        lastReconnectAttempt = 0;
      }
    }
    return;
  }
  
  //Leitura das entradas digitais
  digitalWrite(pin_Latch165, LOW);
  digitalWrite(pin_Latch165, HIGH); 
  for (int i = 0; i <= 7; i++){
    Digital_in[i] = !digitalRead(pin_Data165);
    digitalWrite(pin_Clock165, HIGH);
    digitalWrite(pin_Clock165, LOW);
    Serial.print(Digital_in[i]);
  }
  Digital_in[8] = !digitalRead(pin_X9);
  Digital_in[9] = !digitalRead(pin_X10);
  
  Serial.print(Digital_in[8]);
  Serial.println(Digital_in[9]);
  
  //Leitura das entradas analógicas
  Analog_in[0] = analogRead(pin_AI1);
  Analog_in[1] = analogRead(pin_AI2);
  Analog_in[2] = analogRead(pin_AI3);
    
  Serial.println(Analog_in[0]);
  Serial.println(Analog_in[1]);
  Serial.println(Analog_in[2]);

  /*Construindo a requisição para o Broker*/
  sprintf(topic, "%s", " ");//Insira o Topico entre " "
  sprintf(payload, "%s", ""); // Cleans the payload
  sprintf(payload, "{\"%s\": %i,", VARIABLE_LABEL_1, Digital_in[0]); // Adds the variable label
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_2, Digital_in[1]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_3, Digital_in[2]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_4, Digital_in[3]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_5, Digital_in[4]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_6, Digital_in[5]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_7, Digital_in[6]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_8, Digital_in[7]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_9, Digital_in[8]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_10, Digital_in[9]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_11, Analog_in[0]);
  sprintf(payload, "%s\"%s\": %i,", payload, VARIABLE_LABEL_12, Analog_in[1]); 
  sprintf(payload, "%s\"%s\": %i}", payload, VARIABLE_LABEL_13, Analog_in[2]);


  //Posta a cada 30 segundos
  if (millis() > TMR_POST + 30000){
  Serial.println("Publishing data");
  mqtt.publish(topic, payload);
  mqtt.loop();
  TMR_POST = millis(); 
  }
 
}

void mqttCallback(char* topic, byte* payload, unsigned int len) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("]: ");
  Serial.write(payload, len);
  Serial.println();
}